package djmil.cordacheckers.gamestate;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.VNode;
import djmil.cordacheckers.contracts.GameInfo;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatedBy;
import net.corda.v5.application.flows.ResponderFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionValidator;

@InitiatedBy(protocol = "gamestate-commit")
public class CommitTrxResponder implements ResponderFlow {

    private final static Logger log = LoggerFactory.getLogger(CommitTrxResponder.class);

    // @CordaInject
    // public MemberLookup memberLookup;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;

    @Suspendable
    @Override
    public void call(FlowSession session) {
        log.info("----> CommitTrxResponder: Session "+session);
        try {
            UtxoTransactionValidator txValidator = trxToValidate -> {
                log.info("----> Checking participants... " + trxToValidate);
                //checkParticipants(session, trxToValidate);

                /*
                 * Other checks / actions ?
                 */

                //log.info("Approval for " + trxToValidate.getId());
            };

            if (utxoLedgerService == null) {
                 log.error("utxoLedgerService is NULL");
                 return; 
            }
            UtxoSignedTransaction finalizedSignedTransaction = this.utxoLedgerService
                    .receiveFinality(session, txValidator)
                    .getTransaction();

            log.info("Finished responder flow - " + finalizedSignedTransaction.getId());
        } catch (Exception e) {
            // Log Corda's specific message about finality validation failure
            log.warn("Exceptionally finished responder flow", e);
            log.warn("Values. session: " + session + " utxoLedgerService: "+ utxoLedgerService );
        }
    }

    // @Suspendable
    // void checkParticipants(FlowSession session, UtxoLedgerTransaction gameStateUtxo) {
    //     final GameInfo info = new GameInfo(gameStateUtxo);
    //     final var conterpartyName = session.getCounterparty();
    //     if (info.issuer.compareTo(conterpartyName) != 0)
    //         throw new CordaRuntimeException("Bad Issuer. Expected: " + conterpartyName + " actual: " + info.issuer);

    //     final var myInfo = memberLookup.myInfo();
    //     if (VNode.isCordaCherckersCustodian(myInfo))
    //         return; // Custodian shall not validate state's counterparty

    //     final var opponentName = info.state.getOpponentName(myInfo.getName()); // throws NotInvolved
    //     if (conterpartyName.compareTo(opponentName) != 0)
    //         throw new CordaRuntimeException("Bad Opponent. Expcted: " + conterpartyName + " axtual: " + opponentName);
    // }

    // public static class ParticipantException extends Exception {
    // public ParticipantException(String role, MemberX500Name expected,
    // MemberX500Name actual) {
    // super("Bad participants. " +role +" role: expected '" +expected +"', actual
    // '" +actual +"'");
    // }
    // }
}
