import './NewGame.css'
import React, { useContext } from 'react';
import { GamesGuideContext } from '../../../context/games';

import DropdownList from '../../../components/DropdownList';
import { Color, WhiteStone, BlackStone } from '../../../components/Checkers';

export default function NewGame({ players, setPlayers }) {
    const newGame = useContext(GamesGuideContext).newGame;

    const [whitePlayer, blackPlayer] = (() => {
        if (newGame.myColor === Color.white)
            return [players.user.name, newGame.opponentName];

        if (newGame.myColor === Color.black)
            return [newGame.opponentName, players.user.name];

        return ['', ''];
    })(); // <<-- Execute!

    /*
     * Name options
     */
    const nameOptions = !players.leaderboard ? [<option key='loading' value='…'>…loading</option>] :
        Object.keys(players.leaderboard).map(name =>
            <option key={name} value={name}>
                {players.user.isCurrentUser(name) ? 'You' : name}
            </option>)

    const whiteOptions = Array(nameOptions);
    whiteOptions.push(<option key='default' value=''>{'white player …'}</option>);

    const blackOptions = Array(nameOptions);
    blackOptions.push(<option key='default' value=''>{'black player …'}</option>);

    /*
     * The Component
     */
    const onSelect = (name, myColor) => {
        if (players.user.isCurrentUser(name))
            setPlayers(newGame.opponentName, myColor);
        else
            setPlayers(name, Color.opposite(myColor));
    }

    return (
        <div className='NewGame'>
            <BlackStone />
            <DropdownList selected={blackPlayer} onSelect={(name) => onSelect(name, Color.black)} optionsList={blackOptions} />
            <i>- vs -</i>
            <DropdownList selected={whitePlayer} onSelect={(name) => onSelect(name, Color.white)} optionsList={whiteOptions} />
            <WhiteStone />
        </div>
    )
}